<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Doubridge_Calculator {
    private $logger;
    private $http;
    private $settings;

    public function __construct( $http, $logger ) {
        $this->logger = $logger;
        $this->http = $http;
        
        $settings = get_option('doubridge_api');

        if ( isset( $settings['host'] ) ) {
            $settings['host'] = trim( $settings['host'], '/' );
        }

        $this->settings = $settings;
    }

    /**
     * Calculate the difference freight rates in different regions
     */
    public function beforeCalculate( $cart ) {
        if ( !defined('WOOCOMMERCE_CART') && !defined('WOOCOMMERCE_CHECKOUT') ) return;
        // check config
        if ( !$this->validate() ) {
            return;
        }
        // check token
        $this->checkToken();
        // default region
        $default_country = get_option( 'woocommerce_default_country' );
        list( $default_country, $default_state ) = explode( ':',  $default_country);
        // dest region
        $dest_country = WC()->customer->get_shipping_country( 'edit' );        
        // goods
        $products = [];
        $items = $cart->get_cart_contents();
        foreach( $items as $item ) {
            $quantity = $item['quantity'];
            $product = $item['data'];
            $product_type = $product->get_type();
            $sku = '';
            if ($product_type == 'simple') {
                $sku = $product->get_sku();
            } else {
                $parent_data = $product->get_parent_data();
                $sku = $parent_data['sku'];
            }   
            
            $products[] = [
                'sku' => $sku,
                'quantity' => $quantity
            ];
        }
        // url
        $api_url = $this->getUrl( '/api/ShippingCost' );
        // header
        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );
        // default shipping fee
        $default_data = array(
            'request_shipping_option' => array(
                'iso_code' => $default_country,
                'product_items' => $products
            )
        );
        $default_data = json_encode( $default_data );
        $default_shipping = $this->http->remotePost( $api_url, $default_data, $headers );
        // dest region shipping fee
        $dest_data = array(
            'request_shipping_option' => array(
                'iso_code' => $dest_country,
                'product_items' => $products
            )
        );
        $dest_data = json_encode( $dest_data );
        $dest_shipping = $this->http->remotePost( $api_url, $dest_data, $headers );
        // difference
        $default_shipping_cost = 0;
        $dest_shipping_cost = 0;
        if ( $default_shipping ) {
            $default_shipping_cost = $default_shipping['response_shipping_options'][0]['shipping_cost'];
        }
        if ( $dest_shipping ) {
            $dest_shipping_cost = $dest_shipping['response_shipping_options'][0]['shipping_cost'];
        }
        $balance = $dest_shipping_cost - $default_shipping_cost;
        // shop currency
        $default_currency = get_option( 'woocommerce_currency' );
        $balance = get_currency_exchange( $default_currency, $balance );
        
        if ( $balance > 0 ) {
            $cart->add_fee( 'Additional Fee', $balance );
        }
    }

    /**
     * Settings validate
     */
    private function validate() {
        if ( !isset( $this->settings['host'] ) || 
            empty( $this->settings['host'] ) ||
            !isset( $this->settings['access_token'] ) || 
            empty( $this->settings['access_token'] ) ) {
            
            $this->logger->write( 'Failing, Not configured!' );
            return false;
        }

        return true;
    }

    /**
     * Api access point
     */
    private function getUrl( $path ) {
        return $this->settings['host'] . $path;
    }

    /**
     * Check token
     */
    private function checkToken() {
        $settings = $this->settings;
        // Update token
        if ( $this->http->isTokenExpired( $settings ) ) {
            $result = $this->http->refreshToken( $settings['host'], $settings['client_id'], $settings['client_secret'], $settings['callback_url'], $settings['refresh_token'] );
            
            if ( $result ) {
                $settings['access_token'] = $result['access_token'];
                $settings['refresh_token'] = $result['refresh_token'];
                $settings['expires_time'] = time() + $result['expires_in'];

                update_option( 'doubridge_api', $settings );

                $this->settings = $settings;
            }
        }
    }
}
