<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Doubridge_Category {
    protected $http;
    protected $logger;
    protected $settings;

    public function __construct( $http, $logger ) {   
        $this->http = $http;     
        $this->logger = $logger;

        $settings = get_option('doubridge_api');

        if ( isset( $settings['host'] ) ) {
            $settings['host'] = trim( $settings['host'], '/' );
        }

        $this->settings = $settings;
    }

    public function create( $id ) {
		// check config
        if ( !$this->validate() ) {
            return;
        }
        // check token
        $this->checkToken();

        if ( !$this->isSynchronizedCategories() ) {
            return $this->synchronizeCategories();
        }        

		$category = get_term( $id, '', ARRAY_A );

		$this->syncAddCategory( $category );
	}

	public function edit( $id ) {
		// check config
        if ( !$this->validate() ) {
            return;
        }
        // check token
        $this->checkToken();

        if ( !$this->isSynchronizedCategories() ) {
            return $this->synchronizeCategories();
        }

		$category = get_term( $id, '', ARRAY_A );

		$sync_cat_id = $this->syncGetGategoryId( $id );

        if ( $sync_cat_id ) {
            $this->syncEditCategory( $category, $sync_cat_id );
        } else {
            $this->syncAddCategory( $category );
        }
	}

	public function delete( $id ) {
		// check config
        if ( !$this->validate() ) {
            return;
        }
        // check token
        $this->checkToken();

        if ( !$this->isSynchronizedCategories() ) {
            return $this->synchronizeCategories();
        }

        $term = get_term( $id, '', ARRAY_A );

        if ( !$term || $term['taxonomy'] != 'product_cat' ) {
            return;
        }

		$categories = $this->getSubCategories( $id );

		foreach ( $categories as $term_id ) {
            $sync_cat_id = $this->syncGetGategoryId( $term_id );
            
            if ( $sync_cat_id ) {
			    $this->syncDeleteCategory( $sync_cat_id );
            }
		}
	}

    // private methods

    private function syncAddCategory( $category ) {
        $data = array(
            'child_site_category' => array(
                'category_id' => $category['term_id'],
                'name' => $category['name'],
                'parent_category_id' => $category['parent'],
                'deleted' => 'false',
                'published' => 'true'
            )
        );
        $data_json = json_encode( $data );

        $url = $this->getUrl( '/api/childsitecategory' );
        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );

        $result = $this->http->remotePost( $url, $data_json, $headers );

        if ( $result ) {    
            return $array_body['child_site_categories'][0]['id'];
        }    
    }

    private function syncEditCategory( $category, $id ) {
        $data = array(
            'child_site_category' => array(
                'category_id' => $category['term_id'],
                'name' => $category['name'],
                'parent_category_id' => $category['parent'],
                'deleted' => 'false',
                'published' => 'true'
            )
        );
        $data_json = json_encode( $data );

        $url = $this->getUrl( '/api/childsitecategory/' . $id );
        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );

        $result = $this->http->remotePut( $url, $data_json, $headers );
        
        if ( $result ) {            
            return $result['child_site_categories'][0]['id'];
        }
    }

    private function syncDeleteCategory( $id ) {
        $url = $this->getUrl( '/api/childsitecategory/' . $id );
        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );

        $result = $this->http->remoteDelete( $url, $headers );

        return $result;
    }

    private function synchronizeCategories() {
		global $wpdb;

		$terms = $wpdb->get_results( "SELECT term_id FROM $wpdb->term_taxonomy WHERE taxonomy = 'product_cat'" );
        
        $categories = [];

		foreach ( (array)$terms as $t ) {
			$categories[] = get_term( $t->term_id, '', ARRAY_A );
        }
        
        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );
 
        foreach ( $categories as $c ) {
            $data = array(
                'child_site_category' => array(
                    'category_id' => $c['term_id'],
                    'name' => $c['name'],
                    'parent_category_id' => $c['parent'],
                    'published' => 'true',
                    'deleted' => 'false'
                )
            );
            $data_json = json_encode( $data );
            
            $url = $this->getUrl( '/api/childsitecategory' );
            
            $this->http->remotePost( $url, $data_json, $headers );
        }
    }

    private function syncGetGategoryId( $category_id ) {
        $categories = $this->syncGetCategories();

        foreach ( $categories as $c ) {
            if ( $c['category_id'] == $category_id ) {
                return $c['id'];
            }
        }

        return false;
    }

    private function syncGetCategories() {
        $url = $this->getUrl( '/api/childsitecategories' );

        $headers = array(
            'Authorization' => 'Bearer ' . $this->settings['access_token']
        );

        $result = $this->http->remoteGet( $url, $headers );

        if ( $result ) {
            return $result['child_site_categories'];
        }

        return array();
    }

    private function isSynchronizedCategories() {
        $categories = $this->syncGetCategories();
        
        return ( count($categories) > 0 );
    }

	private function getSubCategories( $category_id ) {
		static $data = [];

		$cat = get_option( 'product_cat_children' );

		$data[$category_id] = $category_id;

		if ( !isset( $cat[$category_id] ) ) {
			return $data;
		}

		$child = $cat[$category_id];

		foreach ( $child as $cid ) {
			$subs = $this->getSubCategories( $cid );

			foreach ( $subs as $s ) {
				$data[$cid] = $cid;
			}
		}
		
		return $data;
	}

    private function validate() {
        if ( !isset( $this->settings['host'] ) || 
            empty( $this->settings['host'] ) ||
            !isset( $this->settings['access_token'] ) || 
            empty( $this->settings['access_token'] ) ) {
            
            $this->logger->write( 'Failing, Not configured!' );
            return false;
        }

        return true;
    }

    /**
     * Api access point
     */
    private function getUrl( $path ) {
        return $this->settings['host'] . $path;
    }

    /**
     * Check token
     */
    private function checkToken() {
        $settings = $this->settings;
        // Update token
        if ( $this->http->isTokenExpired( $settings ) ) {
            $result = $this->http->refreshToken( $settings['host'], $settings['client_id'], $settings['client_secret'], $settings['callback_url'], $settings['refresh_token'] );
            
            if ( $result ) {
                $settings['access_token'] = $result['access_token'];
                $settings['refresh_token'] = $result['refresh_token'];
                $settings['expires_time'] = time() + $result['expires_in'];

                update_option( 'doubridge_api', $settings );

                $this->settings = $settings;
            }
        }
    }
}
